@@ -1,39 +1,46 @@
+/**
+ * \file src/instr/iwffff.c
+ * \brief InterWave FFFF Format Support
+ * \author Jaroslav Kysela <perex@suse.cz>
+ * \date 1999-2001
+ */
 /*
  *  InterWave FFFF Format Support
  *  Copyright (c) 1999 by Jaroslav Kysela <perex@suse.cz>
  *
  *
  *   This library is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU Library General Public License as
- *   published by the Free Software Foundation; either version 2 of
+ *   it under the terms of the GNU Lesser General Public License as
+ *   published by the Free Software Foundation; either version 2.1 of
  *   the License, or (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU Library General Public License for more details.
+ *   GNU Lesser General Public License for more details.
  *
- *   You should have received a copy of the GNU Library General Public
+ *   You should have received a copy of the GNU Lesser General Public
  *   License along with this library; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  *
  */
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <errno.h>
 #include <sys/stat.h>
 #include <stdio.h>
-#include "asoundlib.h"
-#include <linux/ainstr_iw.h>
+#include "local.h"
+#include <sound/ainstr_iw.h>
 
 /*
  *  defines
  */
 
-#define IW_RAM_FILE		"/proc/asound/%i/gus-ram-%i"
-#define IW_ROM_FILE		"/proc/asound/%i/gus-rom-%i"
+#ifndef DOC_HIDDEN
+
+#define IW_RAM_FILE		"/proc/asound/card%i/gus-ram-%i"
+#define IW_ROM_FILE		"/proc/asound/card%i/gus-rom-%i"
 
 #undef IW_ROM_DEBUG
 
@@ -171,10 +178,10 @@ struct envelope_record {
 #define copyright_header IW_ID_VALUE('C', 'P', 'R', 'T')
 #endif
 
-struct snd_iwffff_handle {
+struct _snd_iwffff_handle {
 	int rom;
 	unsigned char *fff_data;
-	unsigned int fff_size;
+	size_t fff_size;
 	char *fff_filename;
 	char *dat_filename;
 	unsigned int start_addr;
@@ -183,6 +190,8 @@ struct snd_iwffff_handle {
 	unsigned int share_id3;
 };
 
+#endif /* DOC_HIDDEN */
+
 /*
  *  local functions
  */
@@ -199,9 +208,20 @@ static int iwffff_get_rom_header(int card, int bank, iwffff_rom_header_t *header
 		close(fd);
 		return -EIO;
 	}
+	if (lseek(fd, IWFFFF_ROM_HDR_SIZE, SEEK_SET) < 0) {
+	        close(fd);
+		return -errno;
+	}
 	return fd;
 }
 
+/**
+ * \brief Open IWFFFF files
+ * \param handle IWFFFF handle
+ * \param name_fff filename of an FFF (header) file
+ * \param name_dat filename of an DAT (data) file
+ * \return 0 on success otherwise a negative error code
+ */
 int snd_instr_iwffff_open(snd_iwffff_handle_t **handle, const char *name_fff, const char *name_dat)
 {
 	snd_iwffff_handle_t *iwf;
@@ -237,7 +257,7 @@ int snd_instr_iwffff_open(snd_iwffff_handle_t **handle, const char *name_fff, co
 		close(fd);
 		return -ENOMEM;
 	}
-	if (read(fd, iwf->fff_data, iwf->fff_size) != iwf->fff_size) {
+	if (read(fd, iwf->fff_data, iwf->fff_size) != (ssize_t) iwf->fff_size) {
 		free(iwf->fff_data);
 		free(iwf);
 		close(fd);
@@ -254,20 +274,31 @@ int snd_instr_iwffff_open(snd_iwffff_handle_t **handle, const char *name_fff, co
 	return 0;
 }
 
+/**
+ * \brief Open IWFFFF ROM
+ * \param handle IWFFFF handle
+ * \param card card number
+ * \param bank ROM bank number (0-3)
+ * \param file ROM file number
+ * \return 0 on success otherwise a negative \c errno code
+ *
+ * Opens \a file in \a bank in the ROM image of card \a card, and
+ * writes a handle pointer into \a *handle. 
+ */
 int snd_instr_iwffff_open_rom(snd_iwffff_handle_t **handle, int card, int bank, int file)
 {
 	unsigned int next_ffff;
 	struct header ffff;
 	snd_iwffff_handle_t *iwf;
 	iwffff_rom_header_t header;
-	int fd, index;
+	int fd, idx;
 
  	if (handle == NULL)
  		return -EINVAL;
 	*handle = NULL;
-	index = 0;
-	if (bank > 4 || file > 255)
-		return -1; 
+	idx = 0;
+	if (bank > 3 || file > 255)
+		return -ENOENT;
 	fd = iwffff_get_rom_header(card, bank, &header);
 	if (fd < 0)
 		return fd;
@@ -276,9 +307,9 @@ int snd_instr_iwffff_open_rom(snd_iwffff_handle_t **handle, int card, int bank,
 			break;
 		ffff.length = snd_LE_to_host_32(ffff.length);
 		next_ffff = lseek(fd, 0, SEEK_CUR) + ffff.length;
-		if (file == index) {
+		if (file == idx) {
 #ifdef IW_ROM_DEBUG
-			fprintf(stderr, "file header at 0x%x size 0x%x\n", rom_pos - sizeof(ffff), ffff.length);
+			SNDERR("file header at 0x%x size 0x%x\n", rom_pos - sizeof(ffff), ffff.length);
 #endif
 			iwf = malloc(sizeof(*iwf));
 			if (iwf == NULL) {
@@ -292,7 +323,7 @@ int snd_instr_iwffff_open_rom(snd_iwffff_handle_t **handle, int card, int bank,
 				close(fd);
 				return -ENOMEM;
 			}
-			if (read(fd, iwf->fff_data, ffff.length) != ffff.length) {
+			if (read(fd, iwf->fff_data, ffff.length) != (ssize_t) ffff.length) {
 				free(iwf->fff_data);
 				free(iwf);
 				close(fd);
@@ -304,24 +335,47 @@ int snd_instr_iwffff_open_rom(snd_iwffff_handle_t **handle, int card, int bank,
 			*handle = iwf;
 			return 0;
         	}
-		index++;
+		idx++;
 		lseek(fd, SEEK_CUR, next_ffff);
 	}
 	close(fd);
 	return -ENOENT;
 }
 
-int snd_instr_iwffff_close(snd_iwffff_handle_t *iwf)
+/**
+ * \brief Open IWFFFF ROM file
+ * \param handle IWFFFF handle
+ * \param name IWFFFF ROM filename
+ * \param bank ROM bank number (0-3)
+ * \param file ROM file number
+ * \return 0 on success otherwise a negative error code
+ */
+#ifndef DOXYGEN
+int snd_instr_iwffff_open_rom_file(snd_iwffff_handle_t **handle ATTRIBUTE_UNUSED, const char *name ATTRIBUTE_UNUSED, int bank ATTRIBUTE_UNUSED, int file ATTRIBUTE_UNUSED)
+#else
+int snd_instr_iwffff_open_rom_file(snd_iwffff_handle_t **handle, const char *name, int bank, int file)
+#endif
 {
-	if (iwf == NULL)
+	/* TODO */
+	return -ENXIO;
+}
+
+/**
+ * \brief Close and free IWFFFF handle
+ * \param handle IWFFFF handle
+ * \return 0 on success otherwise a negative error code
+ */
+int snd_instr_iwffff_close(snd_iwffff_handle_t *handle)
+{
+	if (handle == NULL)
 		return -EINVAL;
-	if (iwf->dat_filename)
-		free(iwf->dat_filename);
-	if (iwf->fff_filename)
-		free(iwf->fff_filename);
-	if (iwf->fff_data)
-		free(iwf->fff_data);
-	free(iwf);
+	if (handle->dat_filename)
+		free(handle->dat_filename);
+	if (handle->fff_filename)
+		free(handle->fff_filename);
+	if (handle->fff_data)
+		free(handle->fff_data);
+	free(handle);
 	return 0;
 }
 
@@ -346,7 +400,6 @@ static void free_envelope(iwffff_env_t *envp)
 		free(rec);
 		rec = nrec;
 	}
-	free(envp);
 }
 
 static void free_layer(iwffff_layer_t *layer)
@@ -369,6 +422,11 @@ static void free_layer(iwffff_layer_t *layer)
 	free(layer);
 }
 
+/**
+ * \brief Free IWFFFF instrument
+ * \param __instr IWFFFF instrument handle
+ * \return 0 on success otherwise a negative error code
+ */
 int snd_instr_iwffff_free(snd_instr_iwffff_t *__instr)
 {
 	iwffff_instrument_t *instr = (iwffff_instrument_t *)__instr; 
@@ -386,8 +444,7 @@ int snd_instr_iwffff_free(snd_instr_iwffff_t *__instr)
 	return 0;
 }
 
-static char *look_for_id(snd_iwffff_handle_t *iwf ATTRIBUTE_UNUSED,
-			 unsigned char *start,
+static char *look_for_id(snd_iwffff_handle_t *iwf ATTRIBUTE_UNUSED, unsigned char *start,
 			 unsigned char *end, ID id)
 {
 	if (!start)
@@ -403,7 +460,7 @@ static char *look_for_id(snd_iwffff_handle_t *iwf ATTRIBUTE_UNUSED,
 static void copy_modulation(iwffff_lfo_t *glfo, unsigned char *buffer)
 {
 	glfo->freq  = snd_LE_to_host_16(*(((unsigned short *)buffer) + 0/2));
-	glfo->depth = snd_LE_to_host_16(*(((unsigned short *)buffer) + 2/1));
+	glfo->depth = snd_LE_to_host_16(*(((unsigned short *)buffer) + 2/2));
 	glfo->sweep = snd_LE_to_host_16(*(((unsigned short *)buffer) + 4/2));
 	glfo->shape = buffer[6];
 	glfo->delay = buffer[7];
@@ -486,7 +543,7 @@ static int load_iw_wave(snd_iwffff_handle_t *file,
 		close(fd);
 		return -ENOMEM;
 	}
-	if (read(fd, result, size) != size) {
+	if (read(fd, result, size) != (ssize_t) size) {
 		free(*result);
 		*result = NULL;
 		close(fd);
@@ -506,7 +563,7 @@ static int load_iw_patch(snd_iwffff_handle_t *iwf, iwffff_instrument_t *instr,
 	unsigned char *current;
 
 #ifdef IW_ROM_DEBUG
-	fprintf(stderr, "load_iw_patch - nlayers = %i\n", snd_LE_to_host_16(*(((unsigned short *)patch) + 8/2));
+	SNDERR("load_iw_patch - nlayers = %i\n", snd_LE_to_host_16(*(((unsigned short *)patch) + 8/2));
 #endif
 	instr->layer_type = patch[6];
 	instr->exclusion = patch[7];
@@ -609,6 +666,14 @@ static int load_iw_patch(snd_iwffff_handle_t *iwf, iwffff_instrument_t *instr,
 	return 0;
 }
 
+/**
+ * \brief Load IWFFFF instrument
+ * \param iwf IWFFFF handle
+ * \param bank program bank number
+ * \param prg program number
+ * \param __iwffff allocated IWFFFF instrument
+ * \return 0 on success otherwise a negative error code
+ */
 int snd_instr_iwffff_load(snd_iwffff_handle_t *iwf, int bank, int prg, snd_instr_iwffff_t **__iwffff)
 {
 	unsigned char *ptr, *end;
@@ -694,9 +759,9 @@ static int iwffff_size(iwffff_instrument_t *instr)
 
 static void copy_lfo_to_stream(iwffff_xlfo_t *xlfo, iwffff_lfo_t *lfo)
 {
-	xlfo->freq = snd_htoi_16(lfo->freq);
-	xlfo->depth = snd_htoi_16(lfo->depth);
-	xlfo->sweep = snd_htoi_16(lfo->sweep);
+	xlfo->freq = __cpu_to_le16(lfo->freq);
+	xlfo->depth = __cpu_to_le16(lfo->depth);
+	xlfo->sweep = __cpu_to_le16(lfo->sweep);
 	xlfo->shape = lfo->shape;
 	xlfo->delay = lfo->delay;
 	
@@ -721,32 +786,39 @@ static int copy_env_to_stream(iwffff_xenv_t *xenv, iwffff_env_t *env, __u32 styp
 		ptr += sizeof(*xrec);
 		size += sizeof(*xrec);
 		xrec->stype = stype;
-		xrec->nattack = snd_htoi_16(rec->nattack);
-		xrec->nrelease = snd_htoi_16(rec->nrelease);
-		xrec->sustain_offset = snd_htoi_16(rec->sustain_offset);
-		xrec->sustain_rate = snd_htoi_16(rec->sustain_rate);
-		xrec->release_rate = snd_htoi_16(rec->release_rate);
+		xrec->nattack = __cpu_to_le16(rec->nattack);
+		xrec->nrelease = __cpu_to_le16(rec->nrelease);
+		xrec->sustain_offset = __cpu_to_le16(rec->sustain_offset);
+		xrec->sustain_rate = __cpu_to_le16(rec->sustain_rate);
+		xrec->release_rate = __cpu_to_le16(rec->release_rate);
 		xrec->hirange = rec->hirange;
 		point = (iwffff_env_point_t *)(rec + 1);
 		for (idx = 0; idx < xrec->nattack + xrec->nrelease; idx++) {
 			xpoint = (iwffff_xenv_point_t *)ptr;
 			ptr += sizeof(*xpoint);
 			size += sizeof(*xpoint);
-			xpoint->offset = snd_htoi_16(point->offset);
-			xpoint->rate = snd_htoi_16(point->rate);
+			xpoint->offset = __cpu_to_le16(point->offset);
+			xpoint->rate = __cpu_to_le16(point->rate);
 			point++;
 		}
 	}
 	return size;
 }
 
-int snd_instr_iwffff_conv_to_stream(snd_instr_iwffff_t *iwffff,
-				    const char *name,
-				    snd_seq_instr_put_t **__data,
-				    long *__size)
+/**
+ * \brief Convert the IWFFFF instrument to byte stream
+ * \param iwffff IWFFFF instrument handle
+ * \param name instrument name
+ * \param __data Result - allocated byte stream
+ * \param __size Result - size of allocated byte stream
+ * \return 0 on success otherwise a negative error code
+ */
+int snd_instr_iwffff_convert_to_stream(snd_instr_iwffff_t *iwffff,
+				       const char *name,
+				       snd_instr_header_t **__data,
+				       size_t *__size)
 {
-	snd_seq_instr_put_t *put;
-	snd_seq_instr_data_t *data;
+	snd_instr_header_t *put;
 	int size;
 	char *ptr;
 	iwffff_instrument_t *instr;
@@ -761,23 +833,20 @@ int snd_instr_iwffff_conv_to_stream(snd_instr_iwffff_t *iwffff,
 	instr = (iwffff_instrument_t *)iwffff;
 	*__data = NULL;
 	*__size = 0;
-	size = sizeof(*data) + iwffff_size(iwffff);
-	put = (snd_seq_instr_put_t *)malloc(sizeof(*put) + size);
-	if (put == NULL)
+	size = iwffff_size(iwffff);
+	if (snd_instr_header_malloc(&put, size) < 0)
 		return -ENOMEM;
 	/* build header */
-	bzero(put, sizeof(*put));
-	data = &put->data;
 	if (name)
-		strncpy(data->name, name, sizeof(data->name)-1);
-	data->type = SND_SEQ_INSTR_ATYPE_DATA;
-	strcpy(data->data.format, SND_SEQ_INSTR_ID_INTERWAVE);
+		snd_instr_header_set_name(put, name);
+	snd_instr_header_set_type(put, SND_SEQ_INSTR_ATYPE_DATA);
+	snd_instr_header_set_format(put, SND_SEQ_INSTR_ID_INTERWAVE);
 	/* build data section */
-	xinstr = (iwffff_xinstrument_t *)(data + 1);
+	xinstr = (iwffff_xinstrument_t *)snd_instr_header_get_data(put);
 	xinstr->stype = IWFFFF_STRU_INSTR;
-	xinstr->exclusion = snd_htoi_16(instr->exclusion);
-	xinstr->layer_type = snd_htoi_16(instr->layer_type);
-	xinstr->exclusion_group = snd_htoi_16(instr->exclusion_group);
+	xinstr->exclusion = __cpu_to_le16(instr->exclusion);
+	xinstr->layer_type = __cpu_to_le16(instr->layer_type);
+	xinstr->exclusion_group = __cpu_to_le16(instr->exclusion_group);
 	xinstr->effect1 = instr->effect1;
 	xinstr->effect1_depth = instr->effect1_depth;
 	xinstr->effect2 = instr->effect2;
@@ -797,7 +866,7 @@ int snd_instr_iwffff_conv_to_stream(snd_instr_iwffff_t *iwffff,
 		xlayer->attenuation = layer->attenuation;
 		copy_lfo_to_stream(&xlayer->tremolo, &layer->tremolo);
 		copy_lfo_to_stream(&xlayer->vibrato, &layer->vibrato);
-		xlayer->freq_scale = snd_htoi_16(layer->freq_scale);
+		xlayer->freq_scale = __cpu_to_le16(layer->freq_scale);
 		xlayer->freq_center = layer->freq_center;
 		ptr += copy_env_to_stream(&xlayer->penv, &layer->penv, IWFFFF_STRU_ENV_RECP);
 		ptr += copy_env_to_stream(&xlayer->venv, &layer->venv, IWFFFF_STRU_ENV_RECV);
@@ -805,17 +874,17 @@ int snd_instr_iwffff_conv_to_stream(snd_instr_iwffff_t *iwffff,
 			xwave = (iwffff_xwave_t *)ptr;
 			ptr += sizeof(*xwave);
 			xwave->stype = IWFFFF_STRU_WAVE;
-			xwave->share_id[1] = snd_htoi_32(wave->share_id[1]);
-			xwave->share_id[2] = snd_htoi_32(wave->share_id[2]);
-			xwave->share_id[3] = snd_htoi_32(wave->share_id[3]);
-			xwave->share_id[4] = snd_htoi_32(wave->share_id[4]);
-			xwave->format = snd_htoi_32(wave->format);
-			xwave->size = snd_htoi_32(wave->size);
-			xwave->start = snd_htoi_32(wave->start);
-			xwave->loop_start = snd_htoi_32(wave->loop_start);
-			xwave->loop_end = snd_htoi_32(wave->loop_end);
-			xwave->loop_repeat = snd_htoi_32(wave->loop_repeat);
-			xwave->sample_ratio = snd_htoi_32(wave->sample_ratio);
+			xwave->share_id[1] = __cpu_to_le32(wave->share_id[1]);
+			xwave->share_id[2] = __cpu_to_le32(wave->share_id[2]);
+			xwave->share_id[3] = __cpu_to_le32(wave->share_id[3]);
+			xwave->share_id[4] = __cpu_to_le32(wave->share_id[4]);
+			xwave->format = __cpu_to_le32(wave->format);
+			xwave->size = __cpu_to_le32(wave->size);
+			xwave->start = __cpu_to_le32(wave->start);
+			xwave->loop_start = __cpu_to_le32(wave->loop_start);
+			xwave->loop_end = __cpu_to_le32(wave->loop_end);
+			xwave->loop_repeat = __cpu_to_le32(wave->loop_repeat);
+			xwave->sample_ratio = __cpu_to_le32(wave->sample_ratio);
 			xwave->attenuation = wave->attenuation;
 			xwave->low_note = wave->low_note;
 			xwave->high_note = wave->high_note;
@@ -823,19 +892,32 @@ int snd_instr_iwffff_conv_to_stream(snd_instr_iwffff_t *iwffff,
 				memcpy(ptr, wave->address.ptr, wave->size);
 				ptr += wave->size;
 			} else {
-				xwave->offset = snd_htoi_32(wave->address.memory);
+				xwave->offset = __cpu_to_le32(wave->address.memory);
 			}
 		}
 	}
 	/* write result */
 	*__data = put;
-	*__size = size;
+	*__size = sizeof(*put) + size;
 	return 0;
 }
 
-int snd_instr_iwffff_convert_from_stream(snd_seq_instr_get_t *data ATTRIBUTE_UNUSED,
-					 long size ATTRIBUTE_UNUSED,
+/**
+ * \brief Convert the byte stream to IWFFFF instrument
+ * \param data Input - byte stream
+ * \param size Input - size of byte stream
+ * \param iwffff Result - allocated IWFFFF instrument handle
+ * \return 0 on success otherwise a negative error code
+ */
+#ifndef DOXYGEN
+int snd_instr_iwffff_convert_from_stream(snd_instr_header_t *data ATTRIBUTE_UNUSED,
+					 size_t size ATTRIBUTE_UNUSED,
 					 snd_instr_iwffff_t **iwffff ATTRIBUTE_UNUSED)
+#else
+int snd_instr_iwffff_convert_from_stream(snd_instr_header_t *data,
+					 size_t size,
+					 snd_instr_iwffff_t **iwffff)
+#endif
 {
 	/* TODO */
 	return -ENXIO;
