@@ -138,22 +138,20 @@ static void mix_select_callbacks(snd_pcm_direct_t *dmix)
 	char line[255];
 	int smp = 0, mmx = 0;
 	
-	/* safe settings for all i386 CPUs */
-	dmix->u.dmix.mix_areas1 = mix_areas1_smp;
 	/* try to determine, if we have a MMX capable CPU */
 	in = fopen("/proc/cpuinfo", "r");
-	if (in == NULL)
-		return;
-	while (!feof(in)) {
-		fgets(line, sizeof(line), in);
-		if (!strncmp(line, "processor", 9))
-			smp++;
-		else if (!strncmp(line, "flags", 5)) {
-			if (strstr(line, " mmx"))
-				mmx = 1;
+	if (in) {
+		while (!feof(in)) {
+			fgets(line, sizeof(line), in);
+			if (!strncmp(line, "processor", 9))
+				smp++;
+			else if (!strncmp(line, "flags", 5)) {
+				if (strstr(line, " mmx"))
+					mmx = 1;
+			}
 		}
+		fclose(in);
 	}
-	fclose(in);
 	// printf("MMX: %i, SMP: %i\n", mmx, smp);
 	if (mmx) {
 		dmix->u.dmix.mix_areas1 = smp > 1 ? mix_areas1_smp_mmx : mix_areas1_mmx;
@@ -164,12 +162,54 @@ static void mix_select_callbacks(snd_pcm_direct_t *dmix)
 }
 #endif
 
+#ifdef __x86_64__
+#define ADD_AND_SATURATE
+
+#define MIX_AREAS1 mix_areas1
+#define MIX_AREAS2 mix_areas2
+#define LOCK_PREFIX ""
+#include "pcm_dmix_x86_64.h"
+#undef MIX_AREAS1
+#undef MIX_AREAS2
+#undef LOCK_PREFIX
+
+#define MIX_AREAS1 mix_areas1_smp
+#define MIX_AREAS2 mix_areas2_smp
+#define LOCK_PREFIX "lock ; "
+#include "pcm_dmix_x86_64.h"
+#undef MIX_AREAS1
+#undef MIX_AREAS2
+#undef LOCK_PREFIX
+ 
+static void mix_select_callbacks(snd_pcm_direct_t *dmix)
+{
+	FILE *in;
+	char line[255];
+	int smp = 0;
+	
+	/* try to determine, if we have SMP */
+	in = fopen("/proc/cpuinfo", "r");
+	if (in) {
+		while (!feof(in)) {
+			fgets(line, sizeof(line), in);
+			if (!strncmp(line, "processor", 9))
+				smp++;
+		}
+		fclose(in);
+	}
+	// printf("SMP: %i\n", smp);
+	dmix->u.dmix.mix_areas1 = smp > 1 ? mix_areas1_smp : mix_areas1;
+	dmix->u.dmix.mix_areas2 = smp > 1 ? mix_areas2_smp : mix_areas2;
+}
+#endif
+
+
 #ifndef ADD_AND_SATURATE
 #warning Please, recode mix_areas1() routine to your architecture...
 static void mix_areas1(unsigned int size,
 		       volatile signed short *dst, signed short *src,
-		       volatile signed int *sum, unsigned int dst_step,
-		       unsigned int src_step, unsigned int sum_step)
+		       volatile signed int *sum, size_t dst_step,
+		       size_t src_step, size_t sum_step)
 {
 	register signed int sample, old_sample;
 
@@ -198,8 +238,8 @@ static void mix_areas1(unsigned int size,
 #warning Please, recode mix_areas2() routine to your architecture...
 static void mix_areas2(unsigned int size,
 		       volatile signed int *dst, signed int *src,
-		       volatile signed int *sum, unsigned int dst_step,
-		       unsigned int src_step, unsigned int sum_step)
+		       volatile signed int *sum, size_t dst_step,
+		       size_t src_step, size_t sum_step)
 {
 	register signed int sample, old_sample;
 
@@ -376,178 +416,6 @@ static int snd_pcm_dmix_sync_ptr(snd_pcm_t *pcm)
  *  plugin implementation
  */
 
-static int snd_pcm_dmix_nonblock(snd_pcm_t *pcm ATTRIBUTE_UNUSED, int nonblock ATTRIBUTE_UNUSED)
-{
-	/* value is cached for us in pcm->mode (SND_PCM_NONBLOCK flag) */
-	return 0;
-}
-
-static int snd_pcm_dmix_async(snd_pcm_t *pcm, int sig, pid_t pid)
-{
-	snd_pcm_direct_t *dmix = pcm->private_data;
-	return snd_timer_async(dmix->timer, sig, pid);
-}
-
-static int snd_pcm_dmix_poll_revents(snd_pcm_t *pcm, struct pollfd *pfds, unsigned int nfds, unsigned short *revents)
-{
-	snd_pcm_direct_t *dmix = pcm->private_data;
-	unsigned short events;
-	static snd_timer_read_t rbuf[5];	/* can be overwriten by multiple plugins, we don't need the value */
-
-	assert(pfds && nfds == 1 && revents);
-	events = pfds[0].revents;
-	if (events & POLLIN) {
-		events |= POLLOUT;
-		events &= ~POLLIN;
-		/* empty the timer read queue */
-		while (snd_timer_read(dmix->timer, &rbuf, sizeof(rbuf)) == sizeof(rbuf)) ;
-	}
-	*revents = events;
-	return 0;
-}
-
-static int snd_pcm_dmix_info(snd_pcm_t *pcm, snd_pcm_info_t * info)
-{
-	// snd_pcm_direct_t *dmix = pcm->private_data;
-
-	memset(info, 0, sizeof(*info));
-	info->stream = pcm->stream;
-	info->card = -1;
-	/* FIXME: fill this with something more useful: we know the hardware name */
-	strncpy(info->id, pcm->name, sizeof(info->id));
-	strncpy(info->name, pcm->name, sizeof(info->name));
-	strncpy(info->subname, pcm->name, sizeof(info->subname));
-	info->subdevices_count = 1;
-	return 0;
-}
-
-static inline snd_mask_t *hw_param_mask(snd_pcm_hw_params_t *params,
-					snd_pcm_hw_param_t var)
-{
-	return &params->masks[var - SND_PCM_HW_PARAM_FIRST_MASK];
-}
-
-static inline snd_interval_t *hw_param_interval(snd_pcm_hw_params_t *params,
-						snd_pcm_hw_param_t var)
-{
-	return &params->intervals[var - SND_PCM_HW_PARAM_FIRST_INTERVAL];
-}
-
-static int hw_param_interval_refine_one(snd_pcm_hw_params_t *params,
-					snd_pcm_hw_param_t var,
-					snd_pcm_hw_params_t *src)
-{
-	snd_interval_t *i;
-
-	if (!(params->rmask & (1<<var)))	/* nothing to do? */
-		return 0;
-	i = hw_param_interval(params, var);
-	if (snd_interval_empty(i)) {
-		SNDERR("dmix interval %i empty?", (int)var);
-		return -EINVAL;
-	}
-	if (snd_interval_refine(i, hw_param_interval(src, var)))
-		params->cmask |= 1<<var;
-	return 0;
-}
-
-#undef REFINE_DEBUG
-
-static int snd_pcm_dmix_hw_refine(snd_pcm_t *pcm, snd_pcm_hw_params_t *params)
-{
-	snd_pcm_direct_t *dmix = pcm->private_data;
-	snd_pcm_hw_params_t *hw_params = &dmix->shmptr->hw_params;
-	static snd_mask_t access = { .bits = { 
-					(1<<SNDRV_PCM_ACCESS_MMAP_INTERLEAVED) |
-					(1<<SNDRV_PCM_ACCESS_MMAP_NONINTERLEAVED) |
-					(1<<SNDRV_PCM_ACCESS_RW_INTERLEAVED) |
-					(1<<SNDRV_PCM_ACCESS_RW_NONINTERLEAVED),
-					0, 0, 0 } };
-	int err;
-
-#ifdef REFINE_DEBUG
-	snd_output_t *log;
-	snd_output_stdio_attach(&log, stderr, 0);
-	snd_output_puts(log, "DMIX REFINE (begin):\n");
-	snd_pcm_hw_params_dump(params, log);
-#endif
-	if (params->rmask & (1<<SND_PCM_HW_PARAM_ACCESS)) {
-		if (snd_mask_empty(hw_param_mask(params, SND_PCM_HW_PARAM_ACCESS))) {
-			SNDERR("dmix access mask empty?");
-			return -EINVAL;
-		}
-		if (snd_mask_refine(hw_param_mask(params, SND_PCM_HW_PARAM_ACCESS), &access))
-			params->cmask |= 1<<SND_PCM_HW_PARAM_ACCESS;
-	}
-	if (params->rmask & (1<<SND_PCM_HW_PARAM_FORMAT)) {
-		if (snd_mask_empty(hw_param_mask(params, SND_PCM_HW_PARAM_FORMAT))) {
-			SNDERR("dmix format mask empty?");
-			return -EINVAL;
-		}
-		if (snd_mask_refine_set(hw_param_mask(params, SND_PCM_HW_PARAM_FORMAT),
-				        snd_mask_value(hw_param_mask(hw_params, SND_PCM_HW_PARAM_FORMAT))))
-			params->cmask |= 1<<SND_PCM_HW_PARAM_FORMAT;
-	}
-	//snd_mask_none(hw_param_mask(params, SND_PCM_HW_PARAM_SUBFORMAT));
-	if (params->rmask & (1<<SND_PCM_HW_PARAM_CHANNELS)) {
-		if (snd_interval_empty(hw_param_interval(params, SND_PCM_HW_PARAM_CHANNELS))) {
-			SNDERR("dmix channels mask empty?");
-			return -EINVAL;
-		}
-		err = snd_interval_refine_set(hw_param_interval(params, SND_PCM_HW_PARAM_CHANNELS), dmix->channels);
-		if (err < 0)
-			return err;
-	}
-	err = hw_param_interval_refine_one(params, SND_PCM_HW_PARAM_RATE, hw_params);
-	if (err < 0)
-		return err;
-	err = hw_param_interval_refine_one(params, SND_PCM_HW_PARAM_BUFFER_SIZE, hw_params);
-	if (err < 0)
-		return err;
-	err = hw_param_interval_refine_one(params, SND_PCM_HW_PARAM_BUFFER_TIME, hw_params);
-	if (err < 0)
-		return err;
-	err = hw_param_interval_refine_one(params, SND_PCM_HW_PARAM_PERIOD_SIZE, hw_params);
-	if (err < 0)
-		return err;
-	err = hw_param_interval_refine_one(params, SND_PCM_HW_PARAM_PERIOD_TIME, hw_params);
-	if (err < 0)
-		return err;
-	err = hw_param_interval_refine_one(params, SND_PCM_HW_PARAM_PERIODS, hw_params);
-	if (err < 0)
-		return err;
-#ifdef REFINE_DEBUG
-	snd_output_puts(log, "DMIX REFINE (end):\n");
-	snd_pcm_hw_params_dump(params, log);
-	snd_output_close(log);
-#endif
-	return 0;
-}
-
-static int snd_pcm_dmix_hw_params(snd_pcm_t *pcm ATTRIBUTE_UNUSED, snd_pcm_hw_params_t * params ATTRIBUTE_UNUSED)
-{
-	/* values are cached in the pcm structure */
-	
-	return 0;
-}
-
-static int snd_pcm_dmix_hw_free(snd_pcm_t *pcm ATTRIBUTE_UNUSED)
-{
-	/* values are cached in the pcm structure */
-	return 0;
-}
-
-static int snd_pcm_dmix_sw_params(snd_pcm_t *pcm ATTRIBUTE_UNUSED, snd_pcm_sw_params_t * params ATTRIBUTE_UNUSED)
-{
-	/* values are cached in the pcm structure */
-	return 0;
-}
-
-static int snd_pcm_dmix_channel_info(snd_pcm_t *pcm, snd_pcm_channel_info_t * info)
-{
-        return snd_pcm_channel_info_shm(pcm, info, -1);
-}
-
 static int snd_pcm_dmix_status(snd_pcm_t *pcm, snd_pcm_status_t * status)
 {
 	snd_pcm_direct_t *dmix = pcm->private_data;
@@ -750,16 +618,6 @@ static snd_pcm_sframes_t snd_pcm_dmix_readn(snd_pcm_t *pcm ATTRIBUTE_UNUSED, voi
 	return -ENODEV;
 }
 
-static int snd_pcm_dmix_mmap(snd_pcm_t *pcm ATTRIBUTE_UNUSED)
-{
-	return 0;
-}
-
-static int snd_pcm_dmix_munmap(snd_pcm_t *pcm ATTRIBUTE_UNUSED)
-{
-	return 0;
-}
-
 static int snd_pcm_dmix_close(snd_pcm_t *pcm)
 {
 	snd_pcm_direct_t *dmix = pcm->private_data;
@@ -833,18 +691,18 @@ static void snd_pcm_dmix_dump(snd_pcm_t *pcm, snd_output_t *out)
 
 static snd_pcm_ops_t snd_pcm_dmix_ops = {
 	.close = snd_pcm_dmix_close,
-	.info = snd_pcm_dmix_info,
-	.hw_refine = snd_pcm_dmix_hw_refine,
-	.hw_params = snd_pcm_dmix_hw_params,
-	.hw_free = snd_pcm_dmix_hw_free,
-	.sw_params = snd_pcm_dmix_sw_params,
-	.channel_info = snd_pcm_dmix_channel_info,
+	.info = snd_pcm_direct_info,
+	.hw_refine = snd_pcm_direct_hw_refine,
+	.hw_params = snd_pcm_direct_hw_params,
+	.hw_free = snd_pcm_direct_hw_free,
+	.sw_params = snd_pcm_direct_sw_params,
+	.channel_info = snd_pcm_direct_channel_info,
 	.dump = snd_pcm_dmix_dump,
-	.nonblock = snd_pcm_dmix_nonblock,
-	.async = snd_pcm_dmix_async,
-	.poll_revents = snd_pcm_dmix_poll_revents,
-	.mmap = snd_pcm_dmix_mmap,
-	.munmap = snd_pcm_dmix_munmap,
+	.nonblock = snd_pcm_direct_nonblock,
+	.async = snd_pcm_direct_async,
+	.poll_revents = snd_pcm_direct_poll_revents,
+	.mmap = snd_pcm_direct_mmap,
+	.munmap = snd_pcm_direct_munmap,
 };
 
 static snd_pcm_fast_ops_t snd_pcm_dmix_fast_ops = {
@@ -997,6 +855,7 @@ int snd_pcm_dmix_open(snd_pcm_t **pcmp, const char *name,
 		spcm->channels = dmix->shmptr->s.channels;
 		spcm->format = dmix->shmptr->s.format;
 		spcm->boundary = dmix->shmptr->s.boundary;
+		spcm->info = dmix->shmptr->s.info;
 		ret = snd_pcm_mmap(spcm);
 		if (ret < 0) {
 			SNDERR("unable to mmap channels");
