@@ -287,40 +287,40 @@ static void snd_pcm_hooks_dump(snd_pcm_t *pcm, snd_output_t *out)
 }
 
 static snd_pcm_ops_t snd_pcm_hooks_ops = {
-	close: snd_pcm_hooks_close,
-	info: snd_pcm_hooks_info,
-	hw_refine: snd_pcm_hooks_hw_refine,
-	hw_params: snd_pcm_hooks_hw_params,
-	hw_free: snd_pcm_hooks_hw_free,
-	sw_params: snd_pcm_hooks_sw_params,
-	channel_info: snd_pcm_hooks_channel_info,
-	dump: snd_pcm_hooks_dump,
-	nonblock: snd_pcm_hooks_nonblock,
-	async: snd_pcm_hooks_async,
-	mmap: snd_pcm_hooks_mmap,
-	munmap: snd_pcm_hooks_munmap,
+	.close = snd_pcm_hooks_close,
+	.info = snd_pcm_hooks_info,
+	.hw_refine = snd_pcm_hooks_hw_refine,
+	.hw_params = snd_pcm_hooks_hw_params,
+	.hw_free = snd_pcm_hooks_hw_free,
+	.sw_params = snd_pcm_hooks_sw_params,
+	.channel_info = snd_pcm_hooks_channel_info,
+	.dump = snd_pcm_hooks_dump,
+	.nonblock = snd_pcm_hooks_nonblock,
+	.async = snd_pcm_hooks_async,
+	.mmap = snd_pcm_hooks_mmap,
+	.munmap = snd_pcm_hooks_munmap,
 };
 
 static snd_pcm_fast_ops_t snd_pcm_hooks_fast_ops = {
-	status: snd_pcm_hooks_status,
-	state: snd_pcm_hooks_state,
-	hwsync: snd_pcm_hooks_hwsync,
-	delay: snd_pcm_hooks_delay,
-	prepare: snd_pcm_hooks_prepare,
-	reset: snd_pcm_hooks_reset,
-	start: snd_pcm_hooks_start,
-	drop: snd_pcm_hooks_drop,
-	drain: snd_pcm_hooks_drain,
-	pause: snd_pcm_hooks_pause,
-	rewind: snd_pcm_hooks_rewind,
-	forward: snd_pcm_hooks_forward,
-	resume: snd_pcm_hooks_resume,
-	writei: snd_pcm_hooks_writei,
-	writen: snd_pcm_hooks_writen,
-	readi: snd_pcm_hooks_readi,
-	readn: snd_pcm_hooks_readn,
-	avail_update: snd_pcm_hooks_avail_update,
-	mmap_commit: snd_pcm_hooks_mmap_commit,
+	.status = snd_pcm_hooks_status,
+	.state = snd_pcm_hooks_state,
+	.hwsync = snd_pcm_hooks_hwsync,
+	.delay = snd_pcm_hooks_delay,
+	.prepare = snd_pcm_hooks_prepare,
+	.reset = snd_pcm_hooks_reset,
+	.start = snd_pcm_hooks_start,
+	.drop = snd_pcm_hooks_drop,
+	.drain = snd_pcm_hooks_drain,
+	.pause = snd_pcm_hooks_pause,
+	.rewind = snd_pcm_hooks_rewind,
+	.forward = snd_pcm_hooks_forward,
+	.resume = snd_pcm_hooks_resume,
+	.writei = snd_pcm_hooks_writei,
+	.writen = snd_pcm_hooks_writen,
+	.readi = snd_pcm_hooks_readi,
+	.readn = snd_pcm_hooks_readn,
+	.avail_update = snd_pcm_hooks_avail_update,
+	.mmap_commit = snd_pcm_hooks_mmap_commit,
 };
 
 /**
