@@ -591,40 +591,40 @@ static void snd_pcm_meter_dump(snd_pcm_t *pcm, snd_output_t *out)
 }
 
 static snd_pcm_ops_t snd_pcm_meter_ops = {
-	close: snd_pcm_meter_close,
-	info: snd_pcm_meter_info,
-	hw_refine: snd_pcm_meter_hw_refine,
-	hw_params: snd_pcm_meter_hw_params,
-	hw_free: snd_pcm_meter_hw_free,
-	sw_params: snd_pcm_meter_sw_params,
-	channel_info: snd_pcm_meter_channel_info,
-	dump: snd_pcm_meter_dump,
-	nonblock: snd_pcm_meter_nonblock,
-	async: snd_pcm_meter_async,
-	mmap: snd_pcm_meter_mmap,
-	munmap: snd_pcm_meter_munmap,
+	.close = snd_pcm_meter_close,
+	.info = snd_pcm_meter_info,
+	.hw_refine = snd_pcm_meter_hw_refine,
+	.hw_params = snd_pcm_meter_hw_params,
+	.hw_free = snd_pcm_meter_hw_free,
+	.sw_params = snd_pcm_meter_sw_params,
+	.channel_info = snd_pcm_meter_channel_info,
+	.dump = snd_pcm_meter_dump,
+	.nonblock = snd_pcm_meter_nonblock,
+	.async = snd_pcm_meter_async,
+	.mmap = snd_pcm_meter_mmap,
+	.munmap = snd_pcm_meter_munmap,
 };
 
 static snd_pcm_fast_ops_t snd_pcm_meter_fast_ops = {
-	status: snd_pcm_meter_status,
-	state: snd_pcm_meter_state,
-	hwsync: snd_pcm_meter_hwsync,
-	delay: snd_pcm_meter_delay,
-	prepare: snd_pcm_meter_prepare,
-	reset: snd_pcm_meter_reset,
-	start: snd_pcm_meter_start,
-	drop: snd_pcm_meter_drop,
-	drain: snd_pcm_meter_drain,
-	pause: snd_pcm_meter_pause,
-	rewind: snd_pcm_meter_rewind,
-	forward: snd_pcm_meter_forward,
-	resume: snd_pcm_meter_resume,
-	writei: snd_pcm_mmap_writei,
-	writen: snd_pcm_mmap_writen,
-	readi: snd_pcm_mmap_readi,
-	readn: snd_pcm_mmap_readn,
-	avail_update: snd_pcm_meter_avail_update,
-	mmap_commit: snd_pcm_meter_mmap_commit,
+	.status = snd_pcm_meter_status,
+	.state = snd_pcm_meter_state,
+	.hwsync = snd_pcm_meter_hwsync,
+	.delay = snd_pcm_meter_delay,
+	.prepare = snd_pcm_meter_prepare,
+	.reset = snd_pcm_meter_reset,
+	.start = snd_pcm_meter_start,
+	.drop = snd_pcm_meter_drop,
+	.drain = snd_pcm_meter_drain,
+	.pause = snd_pcm_meter_pause,
+	.rewind = snd_pcm_meter_rewind,
+	.forward = snd_pcm_meter_forward,
+	.resume = snd_pcm_meter_resume,
+	.writei = snd_pcm_mmap_writei,
+	.writen = snd_pcm_mmap_writen,
+	.readi = snd_pcm_mmap_readi,
+	.readn = snd_pcm_mmap_readn,
+	.avail_update = snd_pcm_meter_avail_update,
+	.mmap_commit = snd_pcm_meter_mmap_commit,
 };
 
 /**
@@ -1220,13 +1220,13 @@ static void s16_reset(snd_pcm_scope_t *scope)
 }
 
 snd_pcm_scope_ops_t s16_ops = {
-	enable: s16_enable,
-	disable: s16_disable,
-	close: s16_close,
-	start: s16_start,
-	stop: s16_stop,
-	update: s16_update,
-	reset: s16_reset,
+	.enable = s16_enable,
+	.disable = s16_disable,
+	.close = s16_close,
+	.start = s16_start,
+	.stop = s16_stop,
+	.update = s16_update,
+	.reset = s16_reset,
 };
 
 #endif
